/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.export;

import java.util.Collections;
import java.util.List;

import de.xoev.xdomea.AbgabeAbgabe0401;
import de.xoev.xdomea.AbgabeAbgabe0401.Schriftgutobjekt;
import de.xoev.xdomea.AkteType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.VorgangType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class XdomeaNachrichtBuilder {

	private VorgangType vorgang;
	private NkAbgabeType kopf;
	private AkteType aktenzeichen;
	private List<DokumentType> representations = Collections.emptyList();
	private List<DokumentType> attachments = Collections.emptyList();
	private List<DokumentType> kommentare = Collections.emptyList();
	private List<DokumentType> postfachMails = Collections.emptyList();
	private List<DokumentType> bescheids = Collections.emptyList();
	private List<HistorienProtokollInformationType> historie = Collections.emptyList();

	public static XdomeaNachrichtBuilder builder() {
		return new XdomeaNachrichtBuilder();
	}

	public XdomeaNachrichtBuilder withKopf(NkAbgabeType kopf) {
		this.kopf = kopf;
		return this;
	}

	public XdomeaNachrichtBuilder withVorgang(VorgangType vorgangType) {
		this.vorgang = vorgangType;
		return this;
	}

	public XdomeaNachrichtBuilder withAktenzeichen(AkteType akteType) {
		this.aktenzeichen = akteType;
		return this;
	}

	public XdomeaNachrichtBuilder withRepresentations(List<DokumentType> representations) {
		this.representations = representations;
		return this;
	}

	public XdomeaNachrichtBuilder withAttachments(List<DokumentType> attachments) {
		this.attachments = attachments;
		return this;
	}

	public XdomeaNachrichtBuilder withHistorie(List<HistorienProtokollInformationType> historie) {
		this.historie = historie;
		return this;
	}

	public XdomeaNachrichtBuilder withKommentare(List<DokumentType> kommentare) {
		this.kommentare = kommentare;
		return this;
	}

	public XdomeaNachrichtBuilder withPostfachMails(List<DokumentType> postfachMails) {
		this.postfachMails = postfachMails;
		return this;
	}

	public XdomeaNachrichtBuilder withBescheids(List<DokumentType> bescheids) {
		this.bescheids = bescheids;
		return this;
	}

	public AbgabeAbgabe0401 build() {
		addVorgangDokumente();
		addVorgangChangeHistory();
		return createAbgabeType();
	}

	void addVorgangDokumente() {
		representations.forEach(vorgang.getDokument()::add);
		attachments.forEach(vorgang.getDokument()::add);
		kommentare.forEach(vorgang.getDokument()::add);
		postfachMails.forEach(vorgang.getDokument()::add);
		bescheids.forEach(vorgang.getDokument()::add);
	}

	void addVorgangChangeHistory() {
		historie.forEach(vorgang.getHistorienProtokollInformation()::add);
	}

	AbgabeAbgabe0401 createAbgabeType() {
		var abgabeType = new AbgabeAbgabe0401();
		abgabeType.setKopf(kopf);
		abgabeType.getSchriftgutobjekt().add(createSchriftgutobjektVorgang());
		abgabeType.getSchriftgutobjekt().add(createSchriftgutobjektAkte());
		return abgabeType;
	}

	private Schriftgutobjekt createSchriftgutobjektVorgang() {
		var schriftgutobjekt = new Schriftgutobjekt();
		schriftgutobjekt.setVorgang(vorgang);
		return schriftgutobjekt;
	}

	private Schriftgutobjekt createSchriftgutobjektAkte() {
		var schriftgutobjekt = new Schriftgutobjekt();
		schriftgutobjekt.setAkte(aktenzeichen);
		return schriftgutobjekt;
	}
}
