package de.ozgcloud.archive.quittung;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandService;
import de.ozgcloud.archive.common.xta.XtaFileHelper;
import de.ozgcloud.archive.common.xta.XtaFileParser;
import de.ozgcloud.archive.common.xta.XtaFileType;
import de.ozgcloud.archive.common.xta.XtaImportConfirmationHandler.XtaAbgabeAdapter;
import de.ozgcloud.archive.common.xta.XtaService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandExecutedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.xta.client.model.XtaFile;
import de.ozgcloud.xta.client.model.XtaMessage;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RequiredArgsConstructor
@Service
class QuittungService {

	static final String ARCHIVE_VORGANG_ORDER = "ARCHIVE_VORGANG";

	private final XtaService xtaService;
	private final XtaFileHelper xtaFileHelper;
	private final XtaFileParser xtaFileParser;

	private final CommandService commandService;

	private final ApplicationEventPublisher eventPublisher;

	public void fetchQuittung() {
		xtaService.fetchMessages(this::consumeQuittung);
	}

	void consumeQuittung(XtaMessage message) {
		if (isImportConfirmation(message.messageFile())) {
			xtaFileHelper.initImportConfirmationHandler(message.messageFile()).getAbgaben().forEach(this::handleAbgabe);
		}
	}

	boolean isImportConfirmation(XtaFile xtaFile) {
		return xtaFileParser.isType(xtaFile, XtaFileType.IMPORT_CONFIRMATION);
	}

	void handleAbgabe(XtaAbgabeAdapter abgabeHandler) {
		var vorgangId = abgabeHandler.getVorgangId();
		var pendingCommands = findPendingArchiveVorgangCommands(vorgangId);

		if (CollectionUtils.isNotEmpty(pendingCommands)) {
			evaluateAbgabe(abgabeHandler, pendingCommands);
		} else {
			LOG.warn("No pending archive command found for vorgang: %s. Cannot confirm archiving succesful", vorgangId);
		}
	}

	private List<ArchiveManagerCommand> findPendingArchiveVorgangCommands(String vorgangId) {
		return commandService.findPending(vorgangId, ARCHIVE_VORGANG_ORDER).toList();
	}

	void evaluateAbgabe(XtaAbgabeAdapter abgabeHandler, List<ArchiveManagerCommand> pendingCommands) {
		if (pendingCommands.size() > 1) {
			LOG.warn("Multiple pending commands found for vorgang: %s.", abgabeHandler.getVorgangId());
		}
		if (abgabeHandler.isSuccessful()) {
			pendingCommands.forEach(this::publishCommandExecutedEvent);
		} else {
			pendingCommands.forEach(command -> publishCommandFailedEvent(abgabeHandler, command));
		}
	}

	void publishCommandExecutedEvent(Command pendingCommand) {
		eventPublisher.publishEvent(new CommandExecutedEvent(pendingCommand));
	}

	void publishCommandFailedEvent(XtaAbgabeAdapter contentHandler, Command pendingCommand) {
		eventPublisher.publishEvent(new CommandFailedEvent(pendingCommand.getId(), contentHandler.getErrorMessage()));
	}
}
