/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;

import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.file.OzgFileTestFactory;

class PrimaerdokumentTypeBuilderTest {

	private final PrimaerdokumentTypeBuilder builder = PrimaerdokumentTypeBuilder.builder().withOzgFile(OzgFileTestFactory.createWithUniqueId());

	@Nested
	class TestBuild {

		private MockedStatic<ExportFilenameGenerator> exportFilenameGeneratorMockedStatic;

		private static final String ERSTELLER = "Ersteller";
		private static final String GENERATED_FILE_NAME = "file_name";
		private static final ZonedDateTime CREATED_AT = ZonedDateTime.now();
		private final OzgFile ozgFile = OzgFileTestFactory.createWithUniqueId();

		@BeforeEach
		void setUp() {
			exportFilenameGeneratorMockedStatic = mockStatic(ExportFilenameGenerator.class);
			exportFilenameGeneratorMockedStatic.when(() -> ExportFilenameGenerator.generateExportFilename(ozgFile)).thenReturn(GENERATED_FILE_NAME);
		}

		@AfterEach
		void tearDown() {
			exportFilenameGeneratorMockedStatic.close();
		}

		@Test
		void shouldGenerateExportFilename() {
			builder.withOzgFile(ozgFile).build();

			exportFilenameGeneratorMockedStatic.verify(() -> ExportFilenameGenerator.generateExportFilename(ozgFile));
		}

		@Test
		void shouldHaveDateiname() {
			var primaerdokumentType = builder.withOzgFile(ozgFile).build();

			assertThat(primaerdokumentType.getDateiname()).isEqualTo(GENERATED_FILE_NAME);
		}

		@Test
		void shouldNotHaveDateiname() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getDateiname()).isNull();
		}

		@Test
		void shouldHaveDateinameOriginal() {
			var primaerdokumentType = builder.withOzgFile(ozgFile).build();

			assertThat(primaerdokumentType.getDateinameOriginal()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveErsteller() {
			var primaerdokumentType = builder.withErsteller(ERSTELLER).build();

			assertThat(primaerdokumentType.getErsteller()).isEqualTo(ERSTELLER);
		}

		@Test
		void shouldNotHaveErsteller() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getErsteller()).isNull();
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var primaerdokumentType = builder.withCreatedAt(CREATED_AT).build();

			assertThat(primaerdokumentType.getDatumUhrzeit()).isEqualTo(DateConverter.toXmlGregorianCalendar(CREATED_AT));
		}

		@Test
		void shouldNotHaveDatumUhrzeit() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getDatumUhrzeit()).isNull();
		}

		@Test
		void shouldThrowIllegalgStateException() {
			builder.withOzgFile(null);

			assertThatThrownBy(builder::build).isInstanceOf(IllegalStateException.class);
		}
	}

}