/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedConstruction;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.common.callcontext.CallContextProvider;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.vorgang.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsResponse;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsResponse;

class OzgFileRemoteServiceTest {

	@InjectMocks
	@Spy
	private OzgFileRemoteService service;
	@Mock
	private FileServiceBlockingStub grpcService;
	@Mock
	private OzgFileMapper mapper;
	@Mock
	private CallContextProvider contextProvider;

	@Nested
	class TestGetAttachments {

		private final GrpcGetAttachmentsResponse response = GrpcGetAttachmentsResponseTestFactory.create();
		private final GrpcGetAttachmentsRequest request = GrpcGetAttachmentsRequestTestFactory.create();

		@Mock
		private FileServiceBlockingStub grpcServiceWithInterceptor;

		@BeforeEach
		void mock() {
			doReturn(request).when(service).buildGrpcGetAttachmentsRequest(VorgangWithEingangTestFactory.ID);
			doReturn(grpcServiceWithInterceptor).when(service).getGrpcServiceWithInterceptor();
			when(grpcServiceWithInterceptor.getAttachments(request)).thenReturn(response);
		}

		@Test
		void shouldGetGrpcServiceWithInterceptor() {
			getAttachments();

			verify(service).getGrpcServiceWithInterceptor();
		}

		@Test
		void shouldBuildRequest() {
			getAttachments();

			verify(service).buildGrpcGetAttachmentsRequest(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			getAttachments();

			verify(grpcServiceWithInterceptor).getAttachments(request);
		}

		@Test
		void shouldCallMapper() {
			getAttachments();

			verify(mapper).fromGrpc(GrpcGetAttachmentsResponseTestFactory.GRPC_FILE);
		}

		@Test
		void shouldReturnMappedOzgFile() {
			var mappedOzgFile = OzgFileTestFactory.create();
			when(mapper.fromGrpc(GrpcGetAttachmentsResponseTestFactory.GRPC_FILE)).thenReturn(mappedOzgFile);

			var attachments = getAttachments();

			assertThat(attachments).containsExactly(mappedOzgFile);
		}

		private List<OzgFile> getAttachments() {
			return service.getAttachments(VorgangWithEingangTestFactory.ID).toList();
		}
	}

	@Nested
	class TestBuildGrpcGetAttachmentsRequest {

		@Test
		void shouldReturnRequest() {
			var request = service.buildGrpcGetAttachmentsRequest(VorgangWithEingangTestFactory.ID);

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcGetAttachmentsRequestTestFactory.create());
		}
	}

	@Nested
	class TestGetRepresentations {

		private final GrpcGetRepresentationsResponse response = GrpcGetRepresentationsResponseTestFactory.create();
		private final GrpcGetRepresentationsRequest request = GrpcGetRepresentationsRequestTestFactory.create();

		@Mock
		private FileServiceBlockingStub grpcServiceWithInterceptor;

		@BeforeEach
		void mock() {
			doReturn(grpcServiceWithInterceptor).when(service).getGrpcServiceWithInterceptor();
			doReturn(request).when(service).buildGrpcGetRepresentationsRequest(VorgangWithEingangTestFactory.ID);
			when(grpcServiceWithInterceptor.getRepresentations(request)).thenReturn(response);
		}

		@Test
		void shouldGetGrpcServiceWithInterceptor() {
			getRepresentations();

			verify(service).getGrpcServiceWithInterceptor();
		}

		@Test
		void shouldBuildRequest() {
			getRepresentations();

			verify(service).buildGrpcGetRepresentationsRequest(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			getRepresentations();

			verify(grpcServiceWithInterceptor).getRepresentations(request);
		}

		@Test
		void shouldCallMapper() {
			getRepresentations();

			verify(mapper).fromGrpc(GrpcGetRepresentationsResponseTestFactory.GRPC_FILE);
		}

		@Test
		void shouldReturnMappedOzgFile() {
			var mappedOzgFile = OzgFileTestFactory.create();
			when(mapper.fromGrpc(GrpcGetRepresentationsResponseTestFactory.GRPC_FILE)).thenReturn(mappedOzgFile);

			var representations = getRepresentations();

			assertThat(representations).containsExactly(mappedOzgFile);
		}

		private List<OzgFile> getRepresentations() {
			return service.getRepresentations(VorgangWithEingangTestFactory.ID).toList();
		}
	}

	@Nested
	class TestBuildGrpcGetRepresentationsRequest {

		@Test
		void shouldReturnRequest() {
			var request = service.buildGrpcGetRepresentationsRequest(VorgangWithEingangTestFactory.ID);

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcGetRepresentationsRequestTestFactory.create());
		}
	}

	@Nested
	class TestGetGrpcServiceWithInterceptor {

		private MockedConstruction<OzgCloudCallContextAttachingInterceptor> mockedConstructionInterceptor;
		private OzgCloudCallContextProvider argumentContextProvider;
		@Mock
		private OzgCloudCallContextAttachingInterceptor interceptor;

		@BeforeEach
		void setUpMocks() {
			mockedConstructionInterceptor = mockConstruction(OzgCloudCallContextAttachingInterceptor.class, (mock, context) -> {
				argumentContextProvider = (OzgCloudCallContextProvider) context.arguments().get(0);
				interceptor = mock;
			});
		}

		@AfterEach
		void cleanUp() {
			mockedConstructionInterceptor.close();
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptor() {
			getGrpcServiceWithInterceptor();

			assertThat(mockedConstructionInterceptor.constructed()).hasSize(1);
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptorWithContextProvider() {
			getGrpcServiceWithInterceptor();

			assertThat(argumentContextProvider).isEqualTo(contextProvider);
		}

		@Test
		void shouldSetInterceptor() {
			getGrpcServiceWithInterceptor();

			verify(grpcService).withInterceptors(interceptor);
		}

		@Test
		void shouldReturnServiceStub() {
			when(grpcService.withInterceptors(any(OzgCloudCallContextAttachingInterceptor.class))).thenReturn(grpcService);

			var returnedServiceStub = getGrpcServiceWithInterceptor();

			assertThat(returnedServiceStub).isEqualTo(grpcService);
		}

		private FileServiceBlockingStub getGrpcServiceWithInterceptor() {
			return service.getGrpcServiceWithInterceptor();
		}
	}

}