/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.archive.common.DateConverter;
import de.xoev.xdomea.AntragsdatenFieldType;
import de.xoev.xdomea.AntragsdatenGroupType;
import de.xoev.xdomea.AntragsdatenItemType;
import de.xoev.xdomea.AntragsdatenMultiValueFieldType;
import de.xoev.xdomea.AntragsdatenSingleValueFieldType;
import de.xoev.xdomea.DatatypeType;

class FormDataMapperTest {

	@Spy
	@InjectMocks
	private FormDataMapper formDataMapper;

	@Mock
	private DatatypeMapper datatypeMapper;

	@Nested
	class TestToAntragsdaten {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final List<AntragsdatenFieldType> formDataItems = List.of(AntragsdatenItemTypeTestFactory.createSingleValueField(),
				AntragsdatenItemTypeTestFactory.createSingleValueField());
		private final List<AntragsdatenFieldType> otherDataItems = List.of(AntragsdatenItemTypeTestFactory.createMultiValueField());

		@Test
		void shouldMapFormData() {
			mockMapDataMethodsToReturnNotEmptyResults();

			formDataMapper.toAntragsdaten(vorgang);

			verify(formDataMapper).mapFormData(vorgang);
		}

		@Test
		void shouldMapOtherData() {
			mockMapDataMethodsToReturnNotEmptyResults();

			formDataMapper.toAntragsdaten(vorgang);

			verify(formDataMapper).mapOtherData(vorgang);
		}

		@Test
		void shouldSetItemToEmptyListIfNoDataWasMapped() {
			mockMapDataMethodsToReturnEmptyResults();

			var antragsdaten = formDataMapper.toAntragsdaten(vorgang);

			assertThat(antragsdaten.getItem()).isEmpty();
		}

		@Test
		void shouldAddMappedFormDataToAntragsdaten() {
			mockMapDataMethodsToReturnNotEmptyResults();

			var antragsdaten = formDataMapper.toAntragsdaten(vorgang);

			assertThat(antragsdaten.getItem()).containsAll(formDataItems);
		}

		@Test
		void shouldAddMappedOtherDataToAntragsdaten() {
			mockMapDataMethodsToReturnNotEmptyResults();

			var antragsdaten = formDataMapper.toAntragsdaten(vorgang);

			assertThat(antragsdaten.getItem()).containsAll(otherDataItems);
		}

		private void mockMapDataMethodsToReturnNotEmptyResults() {
			doReturn(formDataItems).when(formDataMapper).mapFormData(vorgang);
			doReturn(otherDataItems).when(formDataMapper).mapOtherData(vorgang);
		}

		private void mockMapDataMethodsToReturnEmptyResults() {
			doReturn(List.of()).when(formDataMapper).mapFormData(vorgang);
			doReturn(List.of()).when(formDataMapper).mapOtherData(vorgang);
		}
	}

	@Nested
	class TestMapFormData {

		private VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final List<AntragsdatenFieldType> mapDataResult = List.of(AntragsdatenItemTypeTestFactory.createSingleValueField(),
				AntragsdatenItemTypeTestFactory.createSingleValueField());

		@Test
		void shouldReturnEmptyOnNullEingang() {
			vorgang = VorgangWithEingangTestFactory.createBuilder().eingang(null).build();

			var antragsdatenItemTypes = formDataMapper.mapFormData(vorgang);

			assertThat(antragsdatenItemTypes).isEmpty();
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnEmptyOnMissingFormData(Map<String, Object> formData) {
			vorgang = VorgangWithEingangTestFactory.createBuilder()
					.eingang(EingangTestFactory.createBuilder().formData(formData).build())
					.build();

			var antragsdatenItemTypes = formDataMapper.mapFormData(vorgang);

			assertThat(antragsdatenItemTypes).isEmpty();
		}

		@Test
		void shouldMapData() {
			doReturn(List.of()).when(formDataMapper).mapData(EingangTestFactory.AS_MAP);

			formDataMapper.mapFormData(vorgang);

			verify(formDataMapper).mapData(EingangTestFactory.AS_MAP);
		}

		@Test
		void shouldReturnMappedData() {
			doReturn(mapDataResult).when(formDataMapper).mapData(vorgang.getEingang().getFormData());

			var antragsdatenItemTypes = formDataMapper.mapFormData(vorgang);

			assertThat(antragsdatenItemTypes).containsExactlyElementsOf(mapDataResult);
		}
	}

	@Nested
	class TestMapOtherData {

		private VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final List<AntragsdatenFieldType> mapDataResult = List.of(AntragsdatenItemTypeTestFactory.createSingleValueField(),
				AntragsdatenItemTypeTestFactory.createSingleValueField());

		@Test
		void shouldReturnEmptyOnNullEingang() {
			vorgang = VorgangWithEingangTestFactory.createBuilder().eingang(null).build();

			var antragsdatenItemTypes = formDataMapper.mapOtherData(vorgang);

			assertThat(antragsdatenItemTypes).isEmpty();
		}

		@Test
		void shouldReturnEmptyOnNullAntragsteller() {
			vorgang = VorgangWithEingangTestFactory.createBuilder()
					.eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
					.build();

			var antragsdatenItemTypes = formDataMapper.mapOtherData(vorgang);

			assertThat(antragsdatenItemTypes).isEmpty();
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnEmptyOnMissingOtherData(Map<String, Object> otherData) {
			vorgang = VorgangWithEingangTestFactory.createBuilder()
					.eingang(EingangTestFactory.createBuilder()
							.antragsteller(AntragstellerTestFactory.createBuilder()
									.otherData(otherData)
									.build())
							.build())
					.build();

			var antragsdatenItemTypes = formDataMapper.mapOtherData(vorgang);

			assertThat(antragsdatenItemTypes).isEmpty();
		}

		@Test
		void shouldMapData() {
			doReturn(List.of()).when(formDataMapper).mapData(AntragstellerTestFactory.OTHER_DATA);

			formDataMapper.mapOtherData(vorgang);

			verify(formDataMapper).mapData(AntragstellerTestFactory.OTHER_DATA);
		}

		@Test
		void shouldReturnMappedData() {
			doReturn(mapDataResult).when(formDataMapper).mapData(AntragstellerTestFactory.OTHER_DATA);

			var antragsdatenItemTypes = formDataMapper.mapOtherData(vorgang);

			assertThat(antragsdatenItemTypes).containsExactlyElementsOf(mapDataResult);
		}
	}

	@Nested
	class TestMapData {

		private final Map<String, Object> originalData = Map.of("orig-key1", "orig-value1", "orig-key2", "orig-value2");
		private final Map<String, Object> extractedData = Map.of("ext-key1", "ext-value1", "ext-key2", "ext-value2");
		private final List<AntragsdatenFieldType> mapDataResult = List.of(AntragsdatenItemTypeTestFactory.createSingleValueField(),
				AntragsdatenItemTypeTestFactory.createSingleValueField());
		private MockedStatic<WrappedValueExtractor> wrappedValueExtractor;

		@BeforeEach
		void init() {
			wrappedValueExtractor = mockStatic(WrappedValueExtractor.class);
			wrappedValueExtractor.when(() -> WrappedValueExtractor.extractWrappedValues(originalData)).thenReturn(extractedData);
			doReturn(mapDataResult).when(formDataMapper).buildItems(extractedData);
		}

		@AfterEach
		void cleanup() {
			wrappedValueExtractor.close();
		}

		@Test
		void shouldCallWrappedValueExtractor() {
			formDataMapper.mapData(originalData);

			wrappedValueExtractor.verify(() -> WrappedValueExtractor.extractWrappedValues(originalData));
		}

		@Test
		void shouldCallBuildItemsWithExtractedData() {
			formDataMapper.mapData(originalData);

			verify(formDataMapper).buildItems(extractedData);
		}

		@Test
		void shouldReturnBuildItemsResult() {
			var mappedData = formDataMapper.mapData(originalData);

			assertThat(mappedData).isEqualTo(mapDataResult);
		}
	}

	@Nested
	class TestBuildItems {

		private static final String FORMDATA_KEY = "key1";
		private static final String SINGLE_VALUE = "value";
		private static final List<Object> COLLECTION_OF_VALUES = List.of("value");
		private static final Map<String, Object> FORMDATA_SUBMAP = Map.of("key2", "value");

		@Mock
		private AntragsdatenItemType expectedAntragsdatenItem;

		@Test
		void shouldReturnEmptyList() {
			var items = formDataMapper.buildItems(Collections.emptyMap());

			assertThat(items).isEmpty();
		}

		@Test
		void shoulAddSubmap() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildAntragsdatenGroup(FORMDATA_KEY, FORMDATA_SUBMAP);

			var items = formDataMapper.buildItems(Map.of(FORMDATA_KEY, FORMDATA_SUBMAP));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}

		@Test
		void shouldAddCollection() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildMultiValueField(FORMDATA_KEY, COLLECTION_OF_VALUES);

			var items = formDataMapper.buildItems(Map.of(FORMDATA_KEY, COLLECTION_OF_VALUES));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}

		@Test
		void shouldAddSingleValue() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildSingleValueField(FORMDATA_KEY, SINGLE_VALUE);

			var items = formDataMapper.buildItems(Map.of(FORMDATA_KEY, SINGLE_VALUE));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}
	}

	@Nested
	class TestBuildAntragsdatenGroup {

		private static final String NAME = "name";
		private static final String FORMDATA_SUBMAP_VALUE = "value";
		private static final Map<String, Object> FORMDATA_SUBMAP = Map.of("key", FORMDATA_SUBMAP_VALUE);
		private final List<AntragsdatenItemType> items = AntragsdatenItemTypeTestFactory.createAntragsdatenItemTypes();

		@Test
		void shouldSetName() {
			mockFormatValue();

			var antragsdatenGroup = formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			assertThat(antragsdatenGroup.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetItems() {
			doReturn(items).when(formDataMapper).buildItems(FORMDATA_SUBMAP);

			var antragsdatenGroup = (AntragsdatenGroupType) formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			assertThat(antragsdatenGroup.getItem()).containsExactlyElementsOf(items);
		}

		@Test
		void shouldCallBuildItems() {
			mockFormatValue();

			formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			verify(formDataMapper).buildItems(FORMDATA_SUBMAP);
		}

		private void mockFormatValue() {
			when(datatypeMapper.from(FORMDATA_SUBMAP_VALUE)).thenReturn(DatatypeType.STRING);
			doReturn(new Object()).when(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_SUBMAP_VALUE);
		}
	}

	@Nested
	class TestBuildMultiValueField {

		private static final String NAME = "name";
		private static final List<String> FORMDATA_COLLECTION = List.of("value1");

		@Test
		void shouldSetName() {
			when(datatypeMapper.from(FORMDATA_COLLECTION)).thenReturn(DatatypeType.STRING);

			var antragsdatenMultiValueFieldType = formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			assertThat(antragsdatenMultiValueFieldType.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetType() {
			when(datatypeMapper.from(FORMDATA_COLLECTION)).thenReturn(DatatypeType.STRING);

			var antragsdatenMultiValueFieldType = (AntragsdatenFieldType) formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			assertThat(antragsdatenMultiValueFieldType.getType()).isEqualTo(DatatypeType.STRING);
		}

		@Test
		void shouldSetValue() {
			var formData = List.of(1, 2);
			when(datatypeMapper.from(formData)).thenReturn(DatatypeType.INTEGER);

			var antragsdatenMultiValueFieldType = (AntragsdatenMultiValueFieldType) formDataMapper.buildMultiValueField(NAME, formData);

			assertThat(antragsdatenMultiValueFieldType.getValue()).containsExactlyElementsOf(formData);
		}

		@Test
		void shouldCallFormatValue() {
			when(datatypeMapper.from(FORMDATA_COLLECTION)).thenReturn(DatatypeType.STRING);

			formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			verify(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_COLLECTION.get(0));
		}
	}

	@Nested
	class TestBuildSingleValueField {

		private static final String NAME = "name";
		private static final String FORMDATA_VALUE = "value";

		@Test
		void shouldSetName() {
			mockFormatValue();
			var antragsdatenSingleValueFieldType = formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetType() {
			when(datatypeMapper.from(FORMDATA_VALUE)).thenReturn(DatatypeType.STRING);

			var antragsdatenSingleValueFieldType = (AntragsdatenFieldType) formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getType()).isEqualTo(DatatypeType.STRING);
		}

		@Test
		void shouldSetValue() {
			mockFormatValue();

			var antragsdatenSingleValueFieldType = (AntragsdatenSingleValueFieldType) formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getValue()).isEqualTo(FORMDATA_VALUE);
		}

		@Test
		void shouldNotSetIfValueIsNull() {
			formDataMapper.buildSingleValueField(NAME, null);

			verify(formDataMapper, never()).formatValue(DatatypeType.STRING, null);
		}

		@Test
		void shouldCallFormatValue() {
			when(datatypeMapper.from(FORMDATA_VALUE)).thenReturn(DatatypeType.STRING);

			formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			verify(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_VALUE);
		}

		private void mockFormatValue() {
			when(datatypeMapper.from(FORMDATA_VALUE)).thenReturn(DatatypeType.STRING);
			doReturn(FORMDATA_VALUE).when(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_VALUE);
		}
	}

	@Nested
	class TestFormatValue {

		@Test
		void shouldFormatStringType() {
			var value = formDataMapper.formatValue(DatatypeType.STRING, 1);

			assertThat(value).isEqualTo("1");
		}

		@Test
		void shouldFormatDateTimeType() {
			try (var dateConverter = mockStatic(DateConverter.class)) {
				var dateTime = ZonedDateTime.now();

				formDataMapper.formatValue(DatatypeType.DATETIME, dateTime);

				dateConverter.verify(() -> DateConverter.toXmlGregorianCalendar(dateTime));
			}
		}

		@Test
		void shouldFormatNull() {
			var value = formDataMapper.formatValue(DatatypeType.STRING, null);

			assertThat(value).isEqualTo("null");
		}
	}
}