/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.vorgang;

import static de.ozgcloud.archive.vorgang.WrappedValueExtractor.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.MockedStatic;

public class WrappedValueExtractorTest {

	@Nested
	class TestExtractWrappedValues {

		private static final String KEY1 = "key1";
		private static final String KEY2 = "key2";
		private static final String KEY3 = "key3";

		@Captor
		private ArgumentCaptor<Object> valueCaptor;
		private final Map<String, Object> originalData = Map.of(KEY1, "value1", KEY2, true, KEY3, 100);
		private final Map<String, Object> extractedData = Map.of(KEY1, "extracted1", KEY2, "extracted2", KEY3, "extracted3");

		@Test
		void shouldCallReplaceWrappedValueWithDirectValue() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);

				WrappedValueExtractor.extractWrappedValues(originalData);

				wrappedValueExtractor.verify(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(valueCaptor.capture()), times(
						originalData.size()));
				assertThat(valueCaptor.getAllValues()).containsExactlyElementsOf(originalData.values());
			}
		}

		@Test
		void shouldReturnMapWithReplacedValues() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(originalData.get(KEY1)))
						.thenReturn(extractedData.get(KEY1));
				wrappedValueExtractor.when(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(originalData.get(KEY2)))
						.thenReturn(extractedData.get(KEY2));
				wrappedValueExtractor.when(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(originalData.get(KEY3)))
						.thenReturn(extractedData.get(KEY3));

				var result = WrappedValueExtractor.extractWrappedValues(originalData);

				assertThat(result).isEqualTo(extractedData);
			}
		}

		private void unmockMethodUnderTest(MockedStatic<WrappedValueExtractor> wrappedValueExtractor) {
			wrappedValueExtractor.when(() -> WrappedValueExtractor.extractWrappedValues(any())).thenCallRealMethod();
		}
	}

	@Nested
	class TestReplaceWrappedValueWithDirectValue {

		private final String simpleValue = "simple value";
		private final Map<String, Object> mapValue = Map.of("key1", "value1", "key2", "value2");
		private final Map<String, Object> wrappedValue = Map.of(WRAPPER_KEY, "value1");

		@Test
		void shouldReturnValue() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);

				var result = WrappedValueExtractor.replaceWrappedValueWithDirectValue(simpleValue);

				assertThat(result).isEqualTo(simpleValue);
			}
		}

		@Test
		void shouldCallIsWrappedValue() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);

				WrappedValueExtractor.replaceWrappedValueWithDirectValue(mapValue);

				wrappedValueExtractor.verify(() -> WrappedValueExtractor.isWrappedValue(mapValue));
			}
		}

		@Test
		void shouldCallItself() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.isWrappedValue(wrappedValue)).thenReturn(true);

				WrappedValueExtractor.replaceWrappedValueWithDirectValue(wrappedValue);

				wrappedValueExtractor.verify(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(wrappedValue.get(WRAPPER_KEY)));
			}
		}

		@Test
		void shouldReturnResultOfCallingItself() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.isWrappedValue(wrappedValue)).thenReturn(true);

				var result = WrappedValueExtractor.replaceWrappedValueWithDirectValue(wrappedValue);

				assertThat(result).isEqualTo(wrappedValue.get(WRAPPER_KEY));
			}
		}

		@Test
		void shouldCallExtractWrappedValues() {
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.isWrappedValue(mapValue)).thenReturn(false);

				WrappedValueExtractor.replaceWrappedValueWithDirectValue(mapValue);

				wrappedValueExtractor.verify(() -> WrappedValueExtractor.extractWrappedValues(mapValue));
			}
		}

		@Test
		void shouldReturnResultOfExtractWrappedValues() {
			final Map<String, Object> extractVrappedValuesResult = Map.of("resultKey", "resultValue");
			try (MockedStatic<WrappedValueExtractor> wrappedValueExtractor = mockStatic(WrappedValueExtractor.class)) {
				unmockMethodUnderTest(wrappedValueExtractor);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.isWrappedValue(mapValue)).thenReturn(false);
				wrappedValueExtractor.when(() -> WrappedValueExtractor.extractWrappedValues(any())).thenReturn(extractVrappedValuesResult);

				var result = WrappedValueExtractor.replaceWrappedValueWithDirectValue(mapValue);

				assertThat(result).isEqualTo(extractVrappedValuesResult);
			}
		}

		private void unmockMethodUnderTest(MockedStatic<WrappedValueExtractor> wrappedValueExtractor) {
			wrappedValueExtractor.when(() -> WrappedValueExtractor.replaceWrappedValueWithDirectValue(any())).thenCallRealMethod();
		}
	}

	@Nested
	class TestIsWrappedValue {

		@Test
		void shouldReturnTrueIfWrapped() {
			var result = WrappedValueExtractor.isWrappedValue(Map.of(WRAPPER_KEY, "value1"));

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseIfHasMultipleEntries() {
			var result = WrappedValueExtractor.isWrappedValue(Map.of(WRAPPER_KEY, "value1", "key2", "value2"));

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseIfNotWrapped() {
			var result = WrappedValueExtractor.isWrappedValue(Map.of("key1", "value1"));

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseIfEmpty() {
			var result = WrappedValueExtractor.isWrappedValue(Map.of());

			assertThat(result).isFalse();
		}
	}
}
